package com.jiuqi.sql;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.sql.DBException;
import java.util.ArrayList;
import java.util.List;

public class SQLPretreatment {
  private String originSQL;
  
  private String sql;
  
  private List params;
  
  private boolean toUpperCase = false;
  
  private boolean ignoreComment = false;
  
  private static final char[] BLANK_SET = new char[] { ' ', '\t', '\r', '\n' };
  
  public SQLPretreatment(String sql) {
    this.originSQL = sql;
    sql = null;
    this.params = new ArrayList();
  }
  
  public void execute() throws DBException {
    reset();
    StringBuffer sqlBuffer = new StringBuffer();
    build(sqlBuffer);
    this.sql = sqlBuffer.toString().trim();
  }
  
  public String getSQL() {
    return this.sql;
  }
  
  public String[] getParamNames() {
    return (String[])this.params.toArray((Object[])new String[0]);
  }
  
  public void setToUpperCase(boolean toUpperCase) {
    this.toUpperCase = toUpperCase;
  }
  
  public boolean getToUpperCase() {
    return this.toUpperCase;
  }
  
  public void setIgnoreComment(boolean ignoreComment) {
    this.ignoreComment = ignoreComment;
  }
  
  public boolean getIgnoreComment() {
    return this.ignoreComment;
  }
  
  private void reset() {
    this.sql = null;
    this.params.clear();
  }
  
  private char convertChar(char ch) {
    return this.toUpperCase ? Character.toUpperCase(ch) : ch;
  }
  
  private void build(StringBuffer buffer) throws DBException {
    CharReader reader = new CharReader(this, null);
    while (!reader.isEnd()) {
      String paramName;
      char ch = reader.read();
      switch (ch) {
        case 'G':
        case 'g':
          if (reader.check(-1, BLANK_SET) && reader.check(0, new char[] { 'o', 'O' }) && reader.check(1, BLANK_SET))
            return; 
          buffer.append(convertChar(ch));
          continue;
        case '\000':
        case ';':
          break;
        case '-':
          if (reader.peek() == '-') {
            reader.read();
            if (!this.ignoreComment)
              buffer.append("--"); 
            gotoNextLine(reader, buffer);
            continue;
          } 
          buffer.append(ch);
          continue;
        case '/':
          if (reader.peek() == '*') {
            reader.read();
            if (!this.ignoreComment)
              buffer.append("/*"); 
            gotoCommentEnd(reader, buffer);
            continue;
          } 
          buffer.append(ch);
          continue;
        case '\'':
          buffer.append(ch);
          if (!readToStringEnd(reader, buffer))
            throw new DBException("SQL语法错误：含有未结束的字符串！"); 
          continue;
        case '?':
          buffer.append(ch);
          paramName = readParamName(reader);
          this.params.add(paramName);
          continue;
      } 
      buffer.append(convertChar(ch));
    } 
  }
  
  private void gotoNextLine(CharReader reader, StringBuffer buffer) {
    while (!reader.isEnd()) {
      char ch = reader.read();
      if (ch == '\r' || ch == '\n') {
        buffer.append(ch);
        break;
      } 
      if (!this.ignoreComment)
        buffer.append(ch); 
    } 
  }
  
  private void gotoCommentEnd(CharReader reader, StringBuffer buffer) {
    while (!reader.isEnd()) {
      char ch = reader.read();
      if (ch == '*' && reader.peek() == '/') {
        reader.read();
        buffer.append(this.ignoreComment ? " " : "*/");
        break;
      } 
      if (!this.ignoreComment)
        buffer.append(ch); 
    } 
  }
  
  private boolean readToStringEnd(CharReader reader, StringBuffer buffer) {
    while (!reader.isEnd()) {
      char ch = reader.read();
      buffer.append(ch);
      if (ch == '\'')
        return true; 
    } 
    return false;
  }
  
  private String readParamName(CharReader reader) {
    StringBuffer paramName = new StringBuffer();
    while (!reader.isEnd()) {
      char ch = reader.peek();
      if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || (
        ch >= '0' && ch <= '9') || ch == '_' || ch == '@') {
        paramName.append(ch);
        reader.read();
        continue;
      } 
      break;
    } 
    return paramName.toString();
  }
}
